@@ -28,12 +28,12 @@ define([
         // document.getElementsByTagName("head")[0].appendChild(link);
         // 컨테이너에서 전달된 callback 함수가 존재하면 실행.
         if (typeof(callback) === 'function') {
-            var uuid = vpCommon.getUUID();
+            var uuid = 'u' + vpCommon.getUUID();
             // 최대 10회 중복되지 않도록 체크
             for (var idx = 0; idx < 10; idx++) {
                 // 이미 사용중인 uuid 인 경우 다시 생성
                 if ($(vpCommon.wrapSelector(vpCommon.formatString(".{0}", uuid))).length > 0) {
-                    uuid = vpCommon.getUUID();
+                    uuid = 'u' + vpCommon.getUUID();
                 }
             }
             $(vpCommon.wrapSelector(vpCommon.formatString("#{0}", vpConst.OPTION_GREEN_ROOM))).find(vpCommon.formatString(".{0}", vpConst.API_OPTION_PAGE)).addClass(uuid);
@@ -67,15 +67,15 @@ define([
         this.uuid = uuid;   // Load html 영역의 uuid.
         // Import 기본 패키지 목록
         this.packageList = [
-            { name:'numpy',     variable:'np'}
-            , { name:'pandas',  variable:'pd'}
+            { library:'numpy',     alias:'np'}
+            , { library:'pandas',  alias:'pd'}
             , { 
-                name:'matplotlib.pyplot', variable:'plt' 
+                library:'matplotlib.pyplot', alias:'plt' 
                 , include: [
                     '%matplotlib inline'
                 ]
             }
-            , { name:'seaborn', variable:'sns'}
+            , { library:'seaborn', alias:'sns'}
         ];
         
         this.package = {
@@ -86,6 +86,7 @@ define([
             code: '',
             input: [
                 {
+                    name: 'vp_importMeta'
                 }
             ],
             variable: [],
@@ -129,21 +130,35 @@ define([
     ImportPackage.prototype.bindImportPackageGrid = function() {
         var that = this;
 
-        this.packageList.forEach((package, i) => {
-            // <select class="vp-add-type">
-            //         <option value="as">as</option>
-            //         <option value="import">import</option>
-            //     </select>
-            var tagTr = $(`<tr><td><input id="vp_library${i}" type="text" class="vp-input m vp-add-library" placeholder="library name" required value="${package.name.toLowerCase()}"/></td>
-            <td><input id="vp_alias${i}" type="text" class="vp-input m vp-add-alias" placeholder="as" value="${package.variable}"/></td>
-            <td><input type="button" class="vp-remove-option w100" style="width:100%;" value="x"></td></tr>`);
-        
-            $(this.wrapSelector("#vp_tblImport tr:last")).before(tagTr);
 
-            // add to package input
-            that.package.input.push({ name: `vp_library${i}`});
-            that.package.input.push({ name: `vp_alias${i}`});
-        });
+        if (this.metadata) {
+            try {
+                var importMeta = JSON.parse(decodeURIComponent(this.metadata.options.find(x=>x.id=='vp_importMeta').value));
+                importMeta && importMeta.forEach((obj, i) => {
+                    var tagTr = $(`<tr><td><input id="vp_library${i}" type="text" class="vp-input m vp-add-library" placeholder="library name" required value="${obj.library.toLowerCase()}"/></td>
+                    <td><input id="vp_alias${i}" type="text" class="vp-input m vp-add-alias" placeholder="as" value="${obj.alias}"/></td>
+                    <td><input type="button" class="vp-remove-option w100" style="width:100%;" value="x"></td></tr>`);
+                
+                    $(this.wrapSelector("#vp_tblImport tr:last")).before(tagTr);
+                });
+            } catch {}
+        } else {
+            this.packageList.forEach((package, i) => {
+                // <select class="vp-add-type">
+                //         <option value="as">as</option>
+                //         <option value="import">import</option>
+                //     </select>
+                var tagTr = $(`<tr><td><input id="vp_library${i}" type="text" class="vp-input m vp-add-library" placeholder="library name" required value="${package.library.toLowerCase()}"/></td>
+                <td><input id="vp_alias${i}" type="text" class="vp-input m vp-add-alias" placeholder="as" value="${package.alias}"/></td>
+                <td><input type="button" class="vp-remove-option w100" style="width:100%;" value="x"></td></tr>`);
+            
+                $(this.wrapSelector("#vp_tblImport tr:last")).before(tagTr);
+    
+                // add to package input
+                // that.package.input.push({ name: `vp_library${i}`});
+                // that.package.input.push({ name: `vp_alias${i}`});
+            });
+        }
 
         // 라이브러리 삭제
         $(document).on("click", this.wrapSelector('.vp-remove-option'), function() {
@@ -191,6 +206,7 @@ define([
         var sbCode = new sb.StringBuilder();
 
         // code generate with library list
+        var importMeta = [];
         var libraryList = $(this.wrapSelector("#vp_tblImport tr:gt(0):not(:last)"));
         for (var idx = 0; idx < libraryList.length; idx++) {
             var pacName = $(libraryList[idx]).find('.vp-add-library').val();
@@ -199,19 +215,26 @@ define([
             if (pacName == "") {
                 continue;
             }
-            sbCode.appendFormatLine("import {0}{1}", pacName, ((pacAlias === undefined || pacAlias === "") ? "" : (" as " + pacAlias)));
+            if (sbCode.toString().trim().length > 0) {
+                sbCode.appendLine();
+            }
+            sbCode.appendFormat("import {0}{1}", pacName, ((pacAlias === undefined || pacAlias === "") ? "" : (" as " + pacAlias)));
 
             this.packageList.forEach(pack => {
-                if (pack.name == pacName) {
+                if (pack.library == pacName) {
                     // if include code exists?
                     if (pack.include != undefined) {
                         pack.include.forEach(code => {
-                            sbCode.appendLine(code);
-                        })
+                            sbCode.appendLine();
+                            sbCode.append(code);
+                        });
                     }
                 }
             })
+
+            importMeta.push({ library: pacName, alias: pacAlias });
         }
+        $(this.wrapSelector('#vp_importMeta')).val(encodeURIComponent(JSON.stringify(importMeta)));
 
         if (addCell) {
             this.cellExecute(sbCode.toString(), exec);
